extern int Pass = 1; // 1 .. 3
extern int BarOffset = 20;

extern int TypeF = 0;
extern double Margin = 10000;

extern string Tmp1 = "-----------";

// 0 .. 2 * DEFAULT
extern int X11 = 100;
extern int X21 = 100;
extern int X31 = 100;
extern int X41 = 100;

extern string Tmp2 = "-----------";

// 0 .. 2 * DEFAULT
extern int X12 = 100;
extern int X22 = 100;
extern int X32 = 100;
extern int X42 = 100;

#define AMOUNT 4
#define VARIANTS 2
#define DEFAULT 100

#define ALPHA 0.001

double Inputs[AMOUNT][VARIANTS], Smokes[VARIANTS];
int Vars[VARIANTS];

int LotDigits;
double MinLot;

int GetLotDigits( string Symb )
{
  int LotDigits = 0;
  int Tmp = 1 / MarketInfo(Symb, MODE_LOTSTEP) + 0.1;

  Tmp /= 10;

  while (Tmp > 0)
  {
    LotDigits++;
    Tmp /= 10;
  }

  return(LotDigits);
}


void init()
{
  int i, j;
  double Sum[VARIANTS];
  double LotStep;

  ArrayInitialize(Sum, 0);

  Inputs[0][0] = X11;
  Inputs[1][0] = X21;
  Inputs[2][0] = X31;
  Inputs[3][0] = X41;

  Inputs[0][1] = X12;
  Inputs[1][1] = X22;
  Inputs[2][1] = X32;
  Inputs[3][1] = X42;

  for (i = 0; i < AMOUNT; i++)
    for (j = 0; j < VARIANTS; j++)
    {
      Inputs[i][j] -= DEFAULT;
      Sum[j] += MathAbs(Inputs[i][j]);
    }

  for (j = 0; j < VARIANTS; j++)
    if (Sum[j] != 0)
      for (i = 0; i < AMOUNT; i++)
        Inputs[i][j] /= Sum[j];
    else
      for (i = 0; i < AMOUNT; i++)
        Inputs[i][j] = 1 / AMOUNT;

  Pass--;

  Margin /= MarketInfo(Symbol(), MODE_MARGINREQUIRED);

  LotDigits = GetLotDigits(Symbol());
  MinLot = MarketInfo(Symbol(), MODE_MINLOT);
  LotStep = MarketInfo(Symbol(), MODE_LOTSTEP);

  if (MinLot > LotStep + ALPHA)
    Print("WARNING: MinLot (", MinLot, ") > LotStep (", LotStep, ")");

  return;
}

double GetPrice( int Offset )
{
  return(Open[Offset]);
}

double Perceptron( int Pass )
{
  int Offset = 0;
  double Tmp, Res = 0, Max = 0;

  for (int i = 0; i < AMOUNT; i++)
  {
    Tmp = GetPrice(Offset);
    Offset += BarOffset;
    Tmp /= GetPrice(Offset);
    Tmp -= 1;

    Res += Inputs[i][Pass] * Tmp;

    if (Tmp < 0)
      Tmp = -Tmp;

    if (Tmp > Max)
      Max = Tmp;
  }

  if (Max != 0)
    Res /= Max; // -1 .. +1

  return(Res);
}

double GetSignal( int Pass )
{
  double PRes1, PRes2;

  if (Pass < VARIANTS)
    return(Perceptron(Pass));

  PRes1 = Perceptron(0);
  PRes2 = Perceptron(1);

  if (PRes1 * PRes2 > 0)
  {
    PRes1 = MathMin(MathAbs(PRes1), MathAbs(PRes2));

    if (PRes2 < 0)
      PRes1 = -PRes1;

    return(PRes1);
  }

  return(0);
}

double LotFunction( int TypeF, double Res )
{
  double Lot;

  switch (TypeF)
  {
    case 0:
      Lot = NormalizeDouble(MathAbs(Margin * Res), LotDigits);

      break;
    case 1:
      Lot = NormalizeDouble(MathAbs(Margin * Res * Res), LotDigits);
  }

  if (Lot < MinLot)
    Lot = 0;
  else if (Res < 0)
    Lot = -Lot;

  return(Lot);
}

int GetOrderTicket( int Type, bool FlagMax )
{
  int Ticket, TicketMax = -1, TicketMin = -1;
  double Max = 0;
  double Min = 9999;
  int Pos = OrdersTotal() - 1;

  while (Pos >= 0)
  {
    OrderSelect(Pos, SELECT_BY_POS);

    if (OrderType() == Type)
    {
      if (OrderLots() > Max) //        
      {
        Max = OrderLots();
        TicketMax = OrderTicket();
      }

      if (OrderLots() < Min)  //      CloseBy ( MinLot)
      {
         Min = OrderLots();
         TicketMin = OrderTicket();
       }
     }

    Pos--;
  }

  if (FlagMax)
    Ticket = TicketMax;
  else
    Ticket = TicketMin;

  if (Ticket > 0)
    OrderSelect(Ticket, SELECT_BY_TICKET);

  return(Ticket);
}

//   
void CloseLock()
{
  int BuyTicket, SellTicket;
  double SellLots, Tmp;
  string Symb;
  bool FlagMax = TRUE;
  bool FlagRepeat = FALSE;

  BuyTicket = GetOrderTicket(OP_BUY, FlagMax);
  SellTicket = GetOrderTicket(OP_SELL, !FlagMax);

  while ((BuyTicket != -1) && (SellTicket != -1))
  {
    SellLots = OrderLots();

    OrderSelect(BuyTicket, SELECT_BY_TICKET);

    Tmp = MathAbs(OrderLots() - SellLots);

    if ((ALPHA < Tmp) && (Tmp < MinLot - ALPHA))
    {
      if (FlagRepeat)
        break;

      FlagRepeat = TRUE;
      FlagMax = !FlagMax;
    }
    else
    {
      if (!OrderCloseBy(BuyTicket, SellTicket))
        return;

      FlagRepeat = FALSE;
    }

    BuyTicket = GetOrderTicket(OP_BUY, FlagMax);
    SellTicket = GetOrderTicket(OP_SELL, !FlagMax);
  }

  return;
}

void SetPosition( double Vol )
{
  static double Lots = 0;
  double NewLot = NormalizeDouble(MathAbs(Vol - Lots), LotDigits);
  int TicketNew;

  if (NewLot < MinLot)
    return;

  if (Vol > Lots)
    OrderSend(Symbol(), OP_BUY, NewLot, Ask, 0, 0, 0);
  else
    OrderSend(Symbol(), OP_SELL, NewLot, Bid, 0, 0, 0);

  CloseLock();

  Lots = Vol;

  return;
}

void start()
{
  static int PrevTime = 0;

  if (PrevTime == Time[0])
    return;

  PrevTime = Time[0];

  SetPosition(LotFunction(TypeF, GetSignal(Pass)));

  return;
}